package cn.iocoder.yudao.module.smartfactory.controller.screendata.factory;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.smartfactory.controller.screendata.factory.vo.*;
import cn.iocoder.yudao.module.smartfactory.service.screendata.ScreenDataService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.annotation.security.PermitAll;
import javax.validation.Valid;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Tag(name = "工厂大屏数据")
@RestController
@RequestMapping("/smartfactory/factory-screen-data")
@Validated
public class FactoryScreenDataController {

    @Resource
    private ScreenDataService screenDataService;

    /**
     *
     * @param type  1: 省，2 市，3区
     * @param adcode
     * @return
     */
    @GetMapping("/getProvincesData")
    @PermitAll
    @Operation(summary = "大屏全局省份/省/市工厂数据(包函工厂基本信息)")
    @OperateLog(enable = false) // 避免 Post 请求被记录操作日志
    public CommonResult<List<ProvincesDataRespVO>> getProvincesData(Integer type, String adcode) {
        List<ProvincesDataRespVO> provincesDatas = screenDataService.getProvincesData(type,adcode) ;
        return success(provincesDatas);
    }

    @GetMapping("/getStaffData")
    @PermitAll
    @Operation(summary = "大屏员工信息数据")
    @OperateLog(enable = false) // 避免 Post 请求被记录操作日志
    public CommonResult<StaffDataRespVO> getStaffData(Long factoryId) {
        StaffDataRespVO staffDataRespVO = screenDataService.getStaffData(factoryId) ;
        return success(staffDataRespVO);
    }

    @GetMapping("/getPropertyData")
    @PermitAll
    @Operation(summary = "大屏资产数据")
    @OperateLog(enable = false) // 避免 Post 请求被记录操作日志
    public CommonResult<PropertyDataRespVO> getPropertyData(Long factoryId) {
        PropertyDataRespVO propertyDataRespVO = screenDataService.getPropertyData(factoryId) ;
        return success(propertyDataRespVO);
    }

    @GetMapping("/getFactoryOperateData")
    @PermitAll
    @Operation(summary = "大屏工厂运营数据")
    @OperateLog(enable = false) // 避免 Post 请求被记录操作日志
    public CommonResult<OperateDataRespVO> getFactoryOperateData(Long factoryId) {
        OperateDataRespVO factoryOperateDataRespVO = screenDataService.getFactoryOperateData(factoryId) ;
        return success(factoryOperateDataRespVO);
    }

    @GetMapping("/getFactoryRollData")
    @PermitAll
    @Operation(summary = "大屏工厂滚动数据")
    @OperateLog(enable = false) // 避免 Post 请求被记录操作日志
    public CommonResult<FactoryRollDataRespVO> getFactoryRollData() {
        FactoryRollDataRespVO factoryRollDataRespVO = screenDataService.getFactoryRollData() ;
        return success(factoryRollDataRespVO);
    }

    @GetMapping("/getFactoryCamerasData")
    @PermitAll
    @Operation(summary = "大屏工厂监控视频数据")
    @OperateLog(enable = false) // 避免 Post 请求被记录操作日志
    public CommonResult<List<FactoryCameraDataRespVO>> getFactoryCamerasData(Long factoryId) {
        List<FactoryCameraDataRespVO> factoryCameraDatas = screenDataService.getFactoryCamerasData(factoryId) ;
        return success(factoryCameraDatas);
    }

    @GetMapping("/getFactoryWeatherInfo")
    @PermitAll
    @Operation(summary = "大屏工厂天气数据")
    @OperateLog(enable = false) // 避免 Post 请求被记录操作日志
    public CommonResult<FactoryWeatherDataRespVO> getFactoryWeatherInfo(Long factoryId) {
        return success(screenDataService.getFactoryWeatherInfo(factoryId));
    }

    /**
     * 大屏工厂打包线数据
     * @param type
     * @param factoryId
     * @return
     */
    @GetMapping("/getFactoryPackLineInfo")
    @PermitAll
    @Operation(summary = "大屏工厂打包线数据")
    @OperateLog(enable = false) // 避免 Post 请求被记录操作日志
    public CommonResult<List<FactoryPackLineDataRespVO>> getFactoryPackLineInfo(Integer type,Long factoryId ) {
        return success(screenDataService.getFactoryPackLineInfo(type,factoryId));
    }

}